@@ -139,22 +139,22 @@ fn test_h264_packet_unmarshal() -> Result<()> {
         "Unmarshal accepted a STAP-A packet with insufficient data"
     );
 
-    pkt.depacketize(&single_payload)?;
+    let payload = pkt.depacketize(&single_payload)?;
     assert_eq!(
-        pkt.payload, single_payload_unmarshaled,
+        payload, single_payload_unmarshaled,
         "Unmarshaling a single payload shouldn't modify the payload"
     );
 
-    avc_pkt.depacketize(&single_payload)?;
+    let payload = avc_pkt.depacketize(&single_payload)?;
     assert_eq!(
-        avc_pkt.payload, single_payload_unmarshaled_avc,
+        payload, single_payload_unmarshaled_avc,
         "Unmarshaling a single payload into avc stream shouldn't modify the payload"
     );
 
     let mut large_payload_result = BytesMut::new();
     for p in &large_payload_packetized {
-        pkt.depacketize(p)?;
-        large_payload_result.put(&*pkt.payload.clone());
+        let payload = pkt.depacketize(p)?;
+        large_payload_result.put(&*payload.clone());
     }
     assert_eq!(
         large_payload_result.freeze(),
@@ -164,8 +164,8 @@ fn test_h264_packet_unmarshal() -> Result<()> {
 
     let mut large_payload_result_avc = BytesMut::new();
     for p in &large_payload_packetized {
-        avc_pkt.depacketize(p)?;
-        large_payload_result_avc.put(&*avc_pkt.payload.clone());
+        let payload = avc_pkt.depacketize(p)?;
+        large_payload_result_avc.put(&*payload.clone());
     }
     assert_eq!(
         large_payload_result_avc.freeze(),
@@ -173,15 +173,15 @@ fn test_h264_packet_unmarshal() -> Result<()> {
         "Failed to unmarshal a large payload into avc stream"
     );
 
-    pkt.depacketize(&single_payload_multi_nalu)?;
+    let payload = pkt.depacketize(&single_payload_multi_nalu)?;
     assert_eq!(
-        pkt.payload, single_payload_multi_nalu_unmarshaled,
+        payload, single_payload_multi_nalu_unmarshaled,
         "Failed to unmarshal a single packet with multiple NALUs"
     );
 
-    avc_pkt.depacketize(&single_payload_multi_nalu)?;
+    let payload = avc_pkt.depacketize(&single_payload_multi_nalu)?;
     assert_eq!(
-        avc_pkt.payload, single_payload_multi_nalu_unmarshaled_avc,
+        payload, single_payload_multi_nalu_unmarshaled_avc,
         "Failed to unmarshal a single packet with multiple NALUs into avc stream"
     );
 
@@ -190,47 +190,71 @@ fn test_h264_packet_unmarshal() -> Result<()> {
 
 #[test]
 fn test_h264_partition_head_checker_is_partition_head() -> Result<()> {
+    let h264 = H264Packet::default();
     let empty_nalu = Bytes::from_static(&[]);
     assert!(
-        !H264PartitionHeadChecker::is_partition_head(&empty_nalu),
+        !h264.is_partition_head(&empty_nalu),
         "empty nalu must not be a partition head"
     );
 
     let single_nalu = Bytes::from_static(&[1, 0]);
     assert!(
-        H264PartitionHeadChecker::is_partition_head(&single_nalu),
+        h264.is_partition_head(&single_nalu),
         "single nalu must be a partition head"
     );
 
     let stapa_nalu = Bytes::from_static(&[STAPA_NALU_TYPE, 0]);
     assert!(
-        H264PartitionHeadChecker::is_partition_head(&stapa_nalu),
+        h264.is_partition_head(&stapa_nalu),
         "stapa nalu must be a partition head"
     );
 
     let fua_start_nalu = Bytes::from_static(&[FUA_NALU_TYPE, FU_START_BITMASK]);
     assert!(
-        H264PartitionHeadChecker::is_partition_head(&fua_start_nalu),
+        h264.is_partition_head(&fua_start_nalu),
         "fua start nalu must be a partition head"
     );
 
     let fua_end_nalu = Bytes::from_static(&[FUA_NALU_TYPE, FU_END_BITMASK]);
     assert!(
-        !H264PartitionHeadChecker::is_partition_head(&fua_end_nalu),
+        !h264.is_partition_head(&fua_end_nalu),
         "fua end nalu must not be a partition head"
     );
 
     let fub_start_nalu = Bytes::from_static(&[FUB_NALU_TYPE, FU_START_BITMASK]);
     assert!(
-        H264PartitionHeadChecker::is_partition_head(&fub_start_nalu),
+        h264.is_partition_head(&fub_start_nalu),
         "fub start nalu must be a partition head"
     );
 
     let fub_end_nalu = Bytes::from_static(&[FUB_NALU_TYPE, FU_END_BITMASK]);
     assert!(
-        !H264PartitionHeadChecker::is_partition_head(&fub_end_nalu),
+        !h264.is_partition_head(&fub_end_nalu),
         "fub end nalu must not be a partition head"
     );
 
     Ok(())
 }
+
+#[test]
+fn test_h264_payloader_payload_sps_and_pps_handling() -> Result<()> {
+    let mut pck = H264Payloader::default();
+    let expected = vec![
+        Bytes::from_static(&[
+            0x78, 0x00, 0x03, 0x07, 0x00, 0x01, 0x00, 0x03, 0x08, 0x02, 0x03,
+        ]),
+        Bytes::from_static(&[0x05, 0x04, 0x05]),
+    ];
+
+    // When packetizing SPS and PPS are emitted with following NALU
+    let res = pck.payload(1500, &Bytes::from_static(&[0x07, 0x00, 0x01]))?;
+    assert!(res.is_empty(), "Generated payload should be empty");
+
+    let res = pck.payload(1500, &Bytes::from_static(&[0x08, 0x02, 0x03]))?;
+    assert!(res.is_empty(), "Generated payload should be empty");
+
+    let actual = pck.payload(1500, &Bytes::from_static(&[0x05, 0x04, 0x05]))?;
+    assert_eq!(actual, expected, "SPS and PPS aren't packed together");
+
+    Ok(())
+}
