@@ -1,12 +1,15 @@
 #[cfg(test)]
 mod packetizer_test;
 
+use crate::error::Result;
 use crate::{extension::abs_send_time_extension::*, header::*, packet::*, sequence::*};
 use util::marshal::{Marshal, MarshalSize};
 
-use anyhow::Result;
+use async_trait::async_trait;
 use bytes::{Bytes, BytesMut};
 use std::fmt;
+use std::future::Future;
+use std::pin::Pin;
 use std::sync::Arc;
 use std::time::SystemTime;
 
@@ -23,9 +26,10 @@ impl Clone for Box<dyn Payloader + Send + Sync> {
 }
 
 /// Packetizer packetizes a payload
+#[async_trait]
 pub trait Packetizer: fmt::Debug {
     fn enable_abs_send_time(&mut self, value: u8);
-    fn packetize(&mut self, payload: &Bytes, samples: u32) -> Result<Vec<Packet>>;
+    async fn packetize(&mut self, payload: &Bytes, samples: u32) -> Result<Vec<Packet>>;
     fn skip_samples(&mut self, skipped_samples: u32);
     fn clone_to(&self) -> Box<dyn Packetizer + Send + Sync>;
 }
@@ -38,10 +42,23 @@ impl Clone for Box<dyn Packetizer + Send + Sync> {
 
 /// Depacketizer depacketizes a RTP payload, removing any RTP specific data from the payload
 pub trait Depacketizer {
-    fn depacketize(&mut self, b: &Bytes) -> Result<()>;
+    fn depacketize(&mut self, b: &Bytes) -> Result<Bytes>;
+
+    /// Checks if the packet is at the beginning of a partition.  This
+    /// should return false if the result could not be determined, in
+    /// which case the caller will detect timestamp discontinuities.
+    fn is_partition_head(&self, payload: &Bytes) -> bool;
+
+    /// Checks if the packet is at the end of a partition.  This should
+    /// return false if the result could not be determined.
+    fn is_partition_tail(&self, marker: bool, payload: &Bytes) -> bool;
 }
 
-pub type FnTimeGen = Arc<dyn (Fn() -> SystemTime) + Send + Sync>;
+//TODO: SystemTime vs Instant?
+// non-monotonic clock vs monotonically non-decreasing clock
+/// FnTimeGen provides current SystemTime
+pub type FnTimeGen =
+    Arc<dyn (Fn() -> Pin<Box<dyn Future<Output = SystemTime> + Send + 'static>>) + Send + Sync>;
 
 #[derive(Clone)]
 pub(crate) struct PacketizerImpl {
@@ -90,12 +107,13 @@ pub fn new_packetizer(
     }
 }
 
+#[async_trait]
 impl Packetizer for PacketizerImpl {
     fn enable_abs_send_time(&mut self, value: u8) {
         self.abs_send_time = value
     }
 
-    fn packetize(&mut self, payload: &Bytes, samples: u32) -> Result<Vec<Packet>> {
+    async fn packetize(&mut self, payload: &Bytes, samples: u32) -> Result<Vec<Packet>> {
         let payloads = self.payloader.payload(self.mtu - 12, payload)?;
         let mut packets = vec![];
         let (mut i, l) = (0, payloads.len());
@@ -121,7 +139,7 @@ impl Packetizer for PacketizerImpl {
 
         if l != 0 && self.abs_send_time != 0 {
             let st = if let Some(fn_time_gen) = &self.time_gen {
-                fn_time_gen()
+                fn_time_gen().await
             } else {
                 SystemTime::now()
             };
